@@ -17,8 +17,9 @@ use vars qw/ @ISA @EXPORT $VERSION/;
 	_is_zero _is_one
 	_is_even _is_odd
 	_check _zero _one _copy _zeros
+        _rsft _lsft
 );
-$VERSION = '0.06';
+$VERSION = '0.07';
 
 # Package to store unsigned big integers in decimal and do math with them
 
@@ -32,17 +33,44 @@ $VERSION = '0.06';
 # USE_MUL: due to problems on certain os (os390, posix-bc) "* 1e-5" is used
 # instead of "/ 1e5" at some places, (marked with USE_MUL). But instead of
 # using the reverse only on problematic machines, I used it everytime to avoid
-# the costly comparisations. This _should_ work everywhere. Thanx Peter Prymmer
+# the costly comparisons. This _should_ work everywhere. Thanx Peter Prymmer
 
 ##############################################################################
 # global constants, flags and accessory
  
 # constants for easier life
 my $nan = 'NaN';
-my $BASE_LEN = 5;
+
+my $BASE_LEN = 7;
 my $BASE = int("1e".$BASE_LEN);		# var for trying to change it to 1e7
-my $RBASE = 1e-5;           		# see USE_MUL
-my $class = 'Math::BigInt::Calc';
+my $RBASE = abs('1e-'.$BASE_LEN);	# see USE_MUL
+
+BEGIN
+  {
+  # Daniel Pfeiffer: determine largest group of digits that is precisely
+  # multipliable with itself plus carry
+  my ($e, $num) = 4;
+  do {
+     $num = ('9' x ++$e) + 0;
+    $num *= $num + 1;
+  } until ($num == $num - 1 or $num - 1 == $num - 2);
+  $BASE_LEN = $e-1;
+  $BASE = int("1e".$BASE_LEN);
+  $RBASE = abs('1e-'.$BASE_LEN);	# see USE_MUL
+  }
+
+# for quering and setting, to debug/benchmark things
+sub _base_len 
+  {
+  my $b = shift;
+  if (defined $b)
+    {
+    $BASE_LEN = $b;
+    $BASE = int("1e".$BASE_LEN);
+    $RBASE = abs('1e-'.$BASE_LEN);	# see USE_MUL
+    }
+  $BASE_LEN;
+  }
 
 ##############################################################################
 # create objects from various representations
@@ -52,12 +80,12 @@ sub _new
   # (string) return ref to num_array
   # Convert a number from string format to internal base 100000 format.
   # Assumes normalized value as input.
-  shift @_ if $_[0] eq $class;
-  my $d = shift;
+  my $d = $_[1];
   # print "_new $d $$d\n";
   my $il = CORE::length($$d)-1;
   # these leaves '00000' instead of int 0 and will be corrected after any op
-  return [ reverse(unpack("a" . ($il%5+1) . ("a5" x ($il/5)), $$d)) ];
+  return [ reverse(unpack("a" . ($il % $BASE_LEN+1) 
+    . ("a$BASE_LEN" x ($il / $BASE_LEN)), $$d)) ];
   }                                                                             
 
 sub _zero
@@ -74,9 +102,7 @@ sub _one
 
 sub _copy
   {
-  shift @_ if $_[0] eq $class;
-  my $x = shift;
-  return [ @$x ];
+  return [ @{$_[1]} ];
   }
 
 ##############################################################################
@@ -87,8 +113,7 @@ sub _str
   # (ref to BINT) return num_str
   # Convert number from internal base 100000 format to string format.
   # internal format is always normalized (no leading zeros, "-0" => "+0")
-  shift @_ if $_[0] eq $class;
-  my $ar = shift;
+  my $ar = $_[1];
   my $ret = "";
   my $l = scalar @$ar;         # number of parts
   return $nan if $l < 1;       # should not happen
@@ -96,10 +121,11 @@ sub _str
   # leading zero parts in internal representation)
   $l --; $ret .= $ar->[$l]; $l--;
   # Interestingly, the pre-padd method uses more time
-  # the old grep variant takes longer (14 to 10 sec)                            
+  # the old grep variant takes longer (14 to 10 sec)
+  my $z = '0' x ($BASE_LEN-1);                            
   while ($l >= 0)
     {
-    $ret .= substr('0000'.$ar->[$l],-5);   # fastest way I could think of
+    $ret .= substr($z.$ar->[$l],-$BASE_LEN); # fastest way I could think of
     $l--;
     }
   return \$ret;
@@ -108,8 +134,7 @@ sub _str
 sub _num
   {
   # Make a number (scalar int/float) from a BigInt object
-  shift @_ if $_[0] eq $class;
-  my $x = shift;
+  my $x = $_[1];
   return $x->[0] if scalar @$x == 1;  # below $BASE
   my $fac = 1;
   my $num = 0;
@@ -126,18 +151,17 @@ sub _num
 sub _add
   {
   # (ref to int_num_array, ref to int_num_array)
-  # routine to add two base 1e5 numbers
+  # routine to add two base 1eX numbers
   # stolen from Knuth Vol 2 Algorithm A pg 231
-  # there are separate routines to add and sub as per Kunth pg 233
+  # there are separate routines to add and sub as per Knuth pg 233
   # This routine clobbers up array x, but not y.
  
-  shift @_ if $_[0] eq $class;
-  my ($x,$y) = @_;
+  my ($c,$x,$y) = @_;
  
   # for each in Y, add Y to X and carry. If after that, something is left in
   # X, foreach in X add carry to X and then return X, carry
   # Trades one "$j++" for having to shift arrays, $j could be made integer
-  # but this would impose a limit to number-length to 2**32.
+  # but this would impose a limit to number-length of 2**32.
   my $i; my $car = 0; my $j = 0;
   for $i (@$y)
     {
@@ -155,10 +179,9 @@ sub _add
 sub _sub
   {
   # (ref to int_num_array, ref to int_num_array)
-  # subtract base 1e5 numbers -- stolen from Knuth Vol 2 pg 232, $x > $y
-  # subtract Y from X (X is always greater/equal!) by modifiyng x in place
-  shift @_ if $_[0] eq $class;
-  my ($sx,$sy,$s) = @_;
+  # subtract base 1eX numbers -- stolen from Knuth Vol 2 pg 232, $x > $y
+  # subtract Y from X (X is always greater/equal!) by modifying x in place
+  my ($c,$sx,$sy,$s) = @_;
  
   my $car = 0; my $i; my $j = 0;
   if (!$s)
@@ -197,42 +220,40 @@ sub _mul
   {
   # (BINT, BINT) return nothing
   # multiply two numbers in internal representation
-  # modifies first arg, second needs not be different from first
-  shift @_ if $_[0] eq $class;
-  my ($xv,$yv) = @_;
+  # modifies first arg, second need not be different from first
+  my ($c,$xv,$yv) = @_;
  
   my @prod = (); my ($prod,$car,$cty,$xi,$yi);
   # since multiplying $x with $x fails, make copy in this case
-  $yv = [@$xv] if "$xv" eq "$yv";
-    # looping through @$y if $xi == 0 is silly! optimize it!
+  $yv = [@$xv] if "$xv" eq "$yv";	# same references?
   for $xi (@$xv)
     {
     $car = 0; $cty = 0;
+
+    # slow variant
+#    for $yi (@$yv)
+#      {
+#      $prod = $xi * $yi + ($prod[$cty] || 0) + $car;
+#      $prod[$cty++] =
+#       $prod - ($car = int($prod * RBASE)) * $BASE;  # see USE_MUL
+#      }
+#    $prod[$cty] += $car if $car; # need really to check for 0?
+#    $xi = shift @prod;
+
+    # faster variant
+    # looping through this if $xi == 0 is silly - so optimize it away!
+    @prod = (), next if $xi == 0;
     for $yi (@$yv)
       {
       $prod = $xi * $yi + ($prod[$cty] || 0) + $car;
+##     this is actually a tad slower
+##        $prod = $prod[$cty]; $prod += ($car + $xi * $yi);	# no ||0 here
       $prod[$cty++] =
-       $prod - ($car = int($prod * 1e-5)) * $BASE;  # see USE_MUL
+       $prod - ($car = int($prod * $RBASE)) * $BASE;  # see USE_MUL
       }
     $prod[$cty] += $car if $car; # need really to check for 0?
     $xi = shift @prod;
     }
-#  for $xi (@$xv)
-#    {
-#    $car = 0; $cty = 0;
-#    # looping through this if $xi == 0 is silly! optimize it!
-#    if (($xi||0) != 0)
-#      {
-#      for $yi (@$yv)
-#        {
-#        $prod = $prod[$cty]; $prod += ($car + $xi * $yi);	# no ||0 here
-#        $prod[$cty++] =
-#         $prod - ($car = int($prod * 1e-5)) * $BASE;  # see USE_MUL
-#        }
-#      }
-#    $prod[$cty] += $car if $car; # need really to check for 0?
-#    $xi = shift @prod;
-#    }
   push @$xv, @prod;
   __strip_zeros($xv);
   # normalize (handled last to save check for $y->is_zero()
@@ -241,11 +262,10 @@ sub _mul
 
 sub _div
   {
-  # ref to array, ref to array, modify first array and return reminder if 
+  # ref to array, ref to array, modify first array and return remainder if 
   # in list context
-  # does no longer handle sign
-  shift @_ if $_[0] eq $class;
-  my ($x,$yorg) = @_;
+  # no longer handles sign
+  my ($c,$x,$yorg) = @_;
   my ($car,$bar,$prd,$dd,$xi,$yi,@q,$v2,$v1);
 
   my (@d,$tmp,$q,$u2,$u1,$u0);
@@ -280,7 +300,7 @@ sub _div
     #warn "oups v1 is 0, u0: $u0 $y->[-2] $y->[-1] l ",scalar @$y,"\n"
     # if $v1 == 0;
     $q = (($u0 == $v1) ? 99999 : int(($u0*$BASE+$u1)/$v1));
-    --$q while ($v2*$q > ($u0*1e5+$u1-$q*$v1)*$BASE+$u2);
+    --$q while ($v2*$q > ($u0*$BASE+$u1-$q*$v1)*$BASE+$u2);
     if ($q)
       {
       ($car, $bar) = (0,0);
@@ -288,14 +308,14 @@ sub _div
         {
         $prd = $q * $y->[$yi] + $car;
         $prd -= ($car = int($prd * $RBASE)) * $BASE;	# see USE_MUL
-	$x->[$xi] += 1e5 if ($bar = (($x->[$xi] -= $prd + $bar) < 0));
+	$x->[$xi] += $BASE if ($bar = (($x->[$xi] -= $prd + $bar) < 0));
 	}
       if ($x->[-1] < $car + $bar) 
         {
         $car = 0; --$q;
 	for ($yi = 0, $xi = $#$x-$#$y-1; $yi <= $#$y; ++$yi,++$xi) 
           {
-	  $x->[$xi] -= 1e5
+	  $x->[$xi] -= $BASE
 	   if ($car = (($x->[$xi] += $y->[$yi] + $car) > $BASE));
 	  }
 	}   
@@ -329,6 +349,98 @@ sub _div
   return $x;
   }
 
+##############################################################################
+# shifts
+
+sub _rsft
+  {
+  my ($c,$x,$y,$n) = @_;
+
+  if ($n != 10)
+    {
+    return;	# we cant do this here, due to now _pow, so signal failure
+    }
+  else
+    {
+    # shortcut (faster) for shifting by 10)
+    # multiples of $BASE_LEN
+    my $dst = 0;				# destination
+    my $src = _num($c,$y);			# as normal int
+    my $rem = $src % $BASE_LEN;			# reminder to shift
+    $src = int($src / $BASE_LEN);		# source
+    if ($rem == 0)
+      {
+      splice (@$x,0,$src);			# even faster, 38.4 => 39.3
+      }
+    else
+      {
+      my $len = scalar @$x - $src;		# elems to go
+      my $vd; my $z = '0'x $BASE_LEN;
+      $x->[scalar @$x] = 0;			# avoid || 0 test inside loop
+      while ($dst < $len)
+        {
+        $vd = $z.$x->[$src];
+        #print "$dst $src '$vd' ";
+        $vd = substr($vd,-$BASE_LEN,$BASE_LEN-$rem);
+        #print "'$vd' ";
+        $src++;
+        $vd = substr($z.$x->[$src],-$rem,$rem) . $vd;
+        #print "'$vd1' ";
+        #print "'$vd'\n";
+        $vd = substr($vd,-$BASE_LEN,$BASE_LEN) if length($vd) > $BASE_LEN;
+        $x->[$dst] = int($vd);
+        $dst++;
+        }
+      splice (@$x,$dst) if $dst > 0;		# kill left-over array elems
+      pop @$x if $x->[-1] == 0;			# kill last element if 0
+      } # else rem == 0
+    }
+  $x;
+  }
+
+sub _lsft
+  {
+  my ($c,$x,$y,$n) = @_;
+
+  if ($n != 10)
+    {
+    return;	# we cant do this here, due to now _pow, so signal failure
+    }
+  else
+    {
+    # shortcut (faster) for shifting by 10) since we are in base 10eX
+    # multiples of $BASE_LEN:
+    my $src = scalar @$x;			# source
+    my $len = _num($c,$y);			# shift-len as normal int
+    my $rem = $len % $BASE_LEN;			# reminder to shift
+    my $dst = $src + int($len/$BASE_LEN);	# destination
+    my $vd;					# further speedup
+    #print "src $src:",$x->[$src]||0," dst $dst:",$v->[$dst]||0," rem $rem\n";
+    $x->[$src] = 0;				# avoid first ||0 for speed
+    my $z = '0' x $BASE_LEN;
+    while ($src >= 0)
+      {
+      $vd = $x->[$src]; $vd = $z.$vd;
+      #print "s $src d $dst '$vd' ";
+      $vd = substr($vd,-$BASE_LEN+$rem,$BASE_LEN-$rem);
+      #print "'$vd' ";
+      $vd .= $src > 0 ? substr($z.$x->[$src-1],-$BASE_LEN,$rem) : '0' x $rem;
+      #print "'$vd' ";
+      $vd = substr($vd,-$BASE_LEN,$BASE_LEN) if length($vd) > $BASE_LEN;
+      #print "'$vd'\n";
+      $x->[$dst] = int($vd);
+      $dst--; $src--;
+      }
+    # set lowest parts to 0
+    while ($dst >= 0) { $x->[$dst--] = 0; }
+    # fix spurios last zero element
+    splice @$x,-1 if $x->[-1] == 0;
+    #print "elems: "; my $i = 0;
+    #foreach (reverse @$v) { print "$i $_ "; $i++; } print "\n";
+    }
+  $x;
+  }
+
 ##############################################################################
 # testing
 
@@ -336,21 +448,22 @@ sub _acmp
   {
   # internal absolute post-normalized compare (ignore signs)
   # ref to array, ref to array, return <0, 0, >0
-  # arrays must have at least on entry, this is not checked for
+  # arrays must have at least one entry; this is not checked for
 
-  shift @_ if $_[0] eq $class;
-  my ($cx, $cy) = @_;
+  my ($c,$cx, $cy) = @_;
 
   #print "$cx $cy\n"; 
   my ($i,$a,$x,$y,$k);
   # calculate length based on digits, not parts
-  $x = _len($cx); $y = _len($cy);
+  $x = _len('',$cx); $y = _len('',$cy);
   # print "length: ",($x-$y),"\n";
-  return $x-$y if ($x - $y);              # if different in length
+  my $lxy = $x - $y;				# if different in length
+  return -1 if $lxy < 0;
+  return 1 if $lxy > 0;
   #print "full compare\n";
   $i = 0; $a = 0;
   # first way takes 5.49 sec instead of 4.87, but has the early out advantage
-  # so grep is slightly faster, but more unflexible. hm. $_ instead if $k
+  # so grep is slightly faster, but more inflexible. hm. $_ instead of $k
   # yields 5.6 instead of 5.5 sec huh?
   # manual way (abort if unequal, good for early ne)
   my $j = scalar @$cx - 1;
@@ -359,7 +472,9 @@ sub _acmp
    # print "$cx->[$j] $cy->[$j] $a",$cx->[$j]-$cy->[$j],"\n";
    last if ($a = $cx->[$j] - $cy->[$j]); $j--;
    }
-  return $a;
+  return 1 if $a > 0;
+  return -1 if $a < 0;
+  return 0;					# equal
   # while it early aborts, it is even slower than the manual variant
   #grep { return $a if ($a = $_ - $cy->[$i++]); } @$cx;
   # grep way, go trough all (bad for early ne)
@@ -370,30 +485,27 @@ sub _acmp
 sub _len
   {
   # computer number of digits in bigint, minus the sign
-  # int() because add/sub leaves sometimes strings (like '00005') instead of
+  # int() because add/sub sometimes leaves strings (like '00005') instead of
   # int ('5') in this place, causing length to fail
-  shift @_ if $_[0] eq $class;
-  my $cx = shift;
+  my $cx = $_[1];
 
-  return (@$cx-1)*5+length(int($cx->[-1]));
+  return (@$cx-1)*$BASE_LEN+length(int($cx->[-1]));
   }
 
 sub _digit
   {
   # return the nth digit, negative values count backward
   # zero is rightmost, so _digit(123,0) will give 3
-  shift @_ if $_[0] eq $class;
-  my $x = shift;
-  my $n = shift || 0; 
+  my ($c,$x,$n) = @_;
 
-  my $len = _len($x);
+  my $len = _len('',$x);
 
   $n = $len+$n if $n < 0;		# -1 last, -2 second-to-last
   $n = abs($n);				# if negative was too big
   $len--; $n = $len if $n > $len;	# n to big?
   
-  my $elem = int($n / 5);		# which array element
-  my $digit = $n % 5;			# which digit in this element
+  my $elem = int($n / $BASE_LEN);	# which array element
+  my $digit = $n % $BASE_LEN;		# which digit in this element
   $elem = '0000'.@$x[$elem];		# get element padded with 0's
   return substr($elem,-$digit-1,1);
   }
@@ -403,20 +515,19 @@ sub _zeros
   # return amount of trailing zeros in decimal
   # check each array elem in _m for having 0 at end as long as elem == 0
   # Upon finding a elem != 0, stop
-  shift @_ if $_[0] eq $class;
-  my $x = shift;
+  my $x = $_[1];
   my $zeros = 0; my $elem;
   foreach my $e (@$x)
     {
     if ($e != 0)
       {
-      $elem = "$e";                            # preserve x
-      $elem =~ s/.*?(0*$)/$1/;                 # strip anything not zero
-      $zeros *= 5;                             # elems * 5
-      $zeros += CORE::length($elem);           # count trailing zeros
-      last;                                    # early out
+      $elem = "$e";				# preserve x
+      $elem =~ s/.*?(0*$)/$1/;			# strip anything not zero
+      $zeros *= $BASE_LEN;			# elems * 5
+      $zeros += CORE::length($elem);		# count trailing zeros
+      last;					# early out
       }
-    $zeros ++;                                 # real else branch: 50% slower!
+    $zeros ++;					# real else branch: 50% slower!
     }
   return $zeros;
   }
@@ -427,32 +538,28 @@ sub _zeros
 sub _is_zero
   {
   # return true if arg (BINT or num_str) is zero (array '+', '0')
-  shift @_ if $_[0] eq $class;
-  my ($x) = shift;
+  my $x = $_[1];
   return (((scalar @$x == 1) && ($x->[0] == 0))) <=> 0;
   }
 
 sub _is_even
   {
   # return true if arg (BINT or num_str) is even
-  shift @_ if $_[0] eq $class;
-  my ($x) = shift;
+  my $x = $_[1];
   return (!($x->[0] & 1)) <=> 0; 
   }
 
 sub _is_odd
   {
   # return true if arg (BINT or num_str) is even
-  shift @_ if $_[0] eq $class;
-  my ($x) = shift;
+  my $x = $_[1];
   return (($x->[0] & 1)) <=> 0; 
   }
 
 sub _is_one
   {
   # return true if arg (BINT or num_str) is one (array '+', '1')
-  shift @_ if $_[0] eq $class;
-  my ($x) = shift;
+  my $x = $_[1];
   return (scalar @$x == 1) && ($x->[0] == 1) <=> 0; 
   }
 
@@ -460,8 +567,6 @@ sub __strip_zeros
   {
   # internal normalization function that strips leading zeros from the array
   # args: ref to array
-  #trace(@_);
-  shift @_ if $_[0] eq $class;
   my $s = shift;
  
   my $cnt = scalar @$s; # get count of parts
@@ -485,9 +590,8 @@ sub __strip_zeros
 sub _check
   {
   # no checks yet, pull it out from the test suite
-  shift @_ if $_[0] eq $class;
+  my $x = $_[1];
 
-  my ($x) = shift;
   return "$x is not a reference" if !ref($x);
 
   # are all parts are valid?
@@ -529,7 +633,7 @@ was rewritten to use library modules for core math routines. Any
 module which follows the same API as this can be used instead by
 using the following call:
 
-	use Math::BigInt Calc => BigNum;
+	use Math::BigInt lib => BigNum;
 
 =head1 EXPORT
 
@@ -549,10 +653,10 @@ the use by Math::BigInt:
 	_add(obj,obj)	Simple addition of two objects
 	_mul(obj,obj)	Multiplication of two objects
 	_div(obj,obj)	Division of the 1st object by the 2nd
-			In list context, returns (result,reminder).
-			NOTE: this is integer math, so there will no
-			fractional part be returned.
-	_sub(obj,obj)	Simple substraction of 1 object from another
+			In list context, returns (result,remainder).
+			NOTE: this is integer math, so no
+			fractional part will be returned.
+	_sub(obj,obj)	Simple subtraction of 1 object from another
 			a third, optional parameter indicates that the params
 			are swapped. In this case, the first param needs to
 			be preserved, while you can destroy the second.
@@ -575,7 +679,7 @@ the use by Math::BigInt:
 
 The following functions are optional, and can be exported if the underlying lib
 has a fast way to do them. If not defined, Math::BigInt will use a pure, but
-slow Perl function as fallback to emulate these:
+slow, Perl function as fallback to emulate these:
 
 	_from_hex(str)	return ref to new object from ref to hexadecimal string
 	_from_bin(str)	return ref to new object from ref to binary string
@@ -592,17 +696,41 @@ slow Perl function as fallback to emulate these:
 	_pow(obj,obj)	return object 1 to the power of object 2
 	_gcd(obj,obj)	return Greatest Common Divisor of two objects
 	
-	_zeros(obj)	return amount of trailing decimal zeros
+	_zeros(obj)	return number of trailing decimal zeros
 
 	_dec(obj)	decrement object by one (input is >= 1)
 	_inc(obj)	increment object by one
 
-Input strings come in as unsigned but with prefix (aka as '123', '0xabc'
+Input strings come in as unsigned but with prefix (i.e. as '123', '0xabc'
 or '0b1101').
 
-Testing of input parameter validity is done by the caller, so you need not to
-worry about underflow (C<_sub()>, C<_dec()>) nor division by zero or similiar
-cases.
+Testing of input parameter validity is done by the caller, so you need not
+worry about underflow (f.i. in C<_sub()>, C<_dec()>) nor about division by
+zero or similar cases.
+
+The first parameter can be modified, that includes the possibility that you
+return a reference to a completely different object instead. Although keeping
+the reference the same is prefered.
+
+Return values are always references to objects or strings. Exceptions are
+C<_lsft()> and C<_rsft()>, which return undef if they can not shift the
+argument. This is used to delegate shifting of bases different than 10 back
+to BigInt, which will use some generic code to calculate the result.
+
+=head1 WRAP YOUR OWN
+
+If you want to port your own favourite c-lib for big numbers to the
+Math::BigInt interface, you can take any of the already existing modules as
+a rough guideline. You should really wrap up the latest BigInt and BigFloat
+testsuites with your module, and replace the following line:
+
+	use Math::BigInt;
+
+by
+
+	use Math::BigInt lib => 'yourlib';
+
+This way you ensure that your library really works 100% within Math::BigInt.
 
 =head1 LICENSE
  
@@ -617,6 +745,7 @@ Seperated from BigInt and shaped API with the help of John Peacock.
 
 =head1 SEE ALSO
 
-L<Math::BigInt>, L<Math::BigFloat>.
+L<Math::BigInt>, L<Math::BigFloat>, L<Math::BigInt::BitVect> and
+L<Math::BigInt::Pari>.
 
 =cut
